@@ -7,6 +7,13 @@ try:
 except NameError:
     FileExistsError = OSError
 
+def mkdir(fn):
+    'Make a directory that will contain the file.'
+    try:
+        os.makedirs(os.path.split(fn)[0])
+    except FileExistsError:
+        pass
+
 class Warehouse:
     def __repr__(self):
         return 'Warehouse(%s)' % repr(self.cachedir)
@@ -19,23 +26,28 @@ class Warehouse:
 
     def __setitem__(self, index, obj):
         fn = self.filename(index)
-        try:
-            os.makedirs(os.path.split(fn)[0])
-        except FileExistsError:
-            pass
+        mkdir(fn)
         with open(fn, 'wb') as fp:
             pickle.dump(obj, fp)
 
     def __getitem__(self, index):
-        with open(self.filename(index), 'rb') as fp:
-            item = pickle.load(fp)
-        return item
+        try:
+            with open(self.filename(index), 'rb') as fp:
+                item = pickle.load(fp)
+        except FileNotFoundError as e:
+            raise KeyError(*e.args)
+        else:
+            return item
 
     def __delitem__(self, index):
         path = self.filename(index)
-        os.remove(path)
-        for path in _reversed_directories(self.cachedir, os.path.split(path)[0]):
-            os.rmdir(path)
+        try:
+            os.remove(path)
+        except FileNotFoundError as e:
+            raise KeyError(*e.args)
+        else:
+            for path in _reversed_directories(self.cachedir, os.path.split(path)[0]):
+                os.rmdir(path)
 
     def __contains__(self, index):
         return os.path.isfile(self.filename(index))
